#include "inttree.h"
#include <stdlib.h>

struct alist {
    inttree *dato;
    struct alist *next, *prev;
};

struct astack {
    struct alist *top;
    int count;
};

struct aqueue {
    struct alist *head;
    int count;
};

struct alist *acreatelist(void)
{
    struct alist *q = malloc(sizeof(struct alist));

    if(!q) {
        fprintf(stderr,"Errore di allocazione nella creazione della lista\n");
        exit(-1);
    }
    q->next = q->prev = q;
    return q;
}

void ainsert(struct alist *p, inttree *elem)
{
	struct alist *q = malloc(sizeof(struct alist));

	if(!q) {
		fprintf(stderr,"Errore nell'allocazione del nuovo elemento\n");
		exit(-1);	
	}
	q->dato = elem;
	q->next = p->next;
	p->next->prev = q;
	p->next = q;
	q->prev = p;
}

void ainsertatend(struct alist *p, inttree *elem)
{
	struct alist *q = malloc(sizeof(struct alist));

	if(!q) {
		fprintf(stderr,"Errore nell'allocazione del nuovo elemento\n");
		exit(-1);	
	}
	q->dato = elem;
	q->prev = p->prev;
	p->prev->next = q;
	p->prev = q;
	q->next = p;
}

void adelete(struct alist *q)
{
	q->prev->next = q->next;
	q->next->prev = q->prev;
	free(q);	
}

void adestroylist(struct alist *p)
{
	while(p->next != p)
		adelete(p->next);
	free(p);
}

struct astack *createastack(void)
{
    struct astack *st = malloc(sizeof(struct astack));

    if(!st) {
        fprintf(stderr,"Errore di allocazione nella creazione dello stack\n");
        exit(-1);
    }
    st->top = acreatelist();
    st->count = 0;
    return st;
}

void destroyastack(struct astack *st)
{
	adestroylist(st->top);
	free(st);
}

void apush(struct astack *st, inttree* elem)
{
    ainsert(st->top, elem);
    st->count++;
}

inttree *apop(struct astack *st)
{
    inttree *e;

    if(!st->count){
        fprintf(stderr,"Errore: pop su stack vuoto\n");
        exit(-2);
    }
    e = st->top->next->dato;
    adelete(st->top->next);
    st->count--;
    return e;
}

char aempty(struct astack *st)
{
    return !st->count;
}

struct aqueue *createaqueue(void)
{
    struct aqueue *q = malloc(sizeof(struct aqueue));

    if(!q) {
        fprintf(stderr,"Errore di allocazione nella creazione della coda\n");
        exit(-1);
    }
    q->head = acreatelist();
    q->count = 0;
    return q;
}

void destroyaqueue(struct aqueue *q)
{
	adestroylist(q->head);
	free(q);
}

void aenqueue(struct aqueue *q, inttree *elem)
{
    ainsertatend(q->head, elem);
    q->count++;
}

inttree *adequeue(struct aqueue *q)
{
    inttree *e;

    if(!q->count){
        fprintf(stderr,"Errore: dequeue su coda vuota\n");
        exit(-2);
    }
    e = q->head->next->dato;
    adelete(q->head->next);
    q->count--;
    return e;
}

char aemptyq(struct aqueue *q)
{
    return !q->count;
}

void preorderit(inttree *p,void (*op)(inttree *))
{
	struct astack *s = createastack();

	apush(s,p);
	while(!aempty(s)) {
		p = apop(s);
		(*op)(p);
		if(p->right)
			apush(s,p->right);
		if(p->left)
			apush(s,p->left);
	}
	destroyastack(s);
}

void levelorderit(inttree *p,void (*op)(inttree *))
{
	struct aqueue *s = createaqueue();

	aenqueue(s,p);
	while(!aemptyq(s)) {
		p = adequeue(s);
		(*op)(p);
		if(p->left)
			aenqueue(s,p->left);
		if(p->right)
			aenqueue(s,p->right);
	}
	destroyaqueue(s);
}

