#include <stdio.h>
#include <stdlib.h>
#include "rbtree.h"

struct spaces {
	int col;
	rbnode *ptr;
	struct spaces *prev, *next;
};

struct spaces *createlist(void)
{
    struct spaces *q = malloc(sizeof(struct spaces));

    if(!q) {
        fprintf(stderr,"Errore di allocazione nella creazione della lista\n");
        exit(-1);
    }
    q->next = q->prev = q;
    return q;
}

void deletelist(struct spaces *q)
{
	q->prev->next = q->next;
	q->next->prev = q->prev;
	free(q);	
}

void destroylist(struct spaces *p)
{
	while(p->next != p)
		deletelist(p->next);
	free(p);
}

void insertatend(struct spaces *p, rbnode *ptr, int col)
{
	struct spaces *q = malloc(sizeof(struct spaces));

	if(!q) {
		fprintf(stderr,"Errore nell'allocazione del nuovo elemento\n");
		exit(-1);	
	}
	q->ptr = ptr;
	q->col = col;
	q->prev = p->prev;
	p->prev->next = q;
	p->prev = q;
	q->next = p;
}

void levelvisit(rbnode *p, rbnode *nil, int lev, struct spaces **sp)
{
	static int count = 0;

	if(!lev) count = 0;
	if(p != nil)	{
		levelvisit(p->left,nil,lev + 1,sp);
		insertatend(sp[lev],p,count++);
		levelvisit(p->right,nil,lev + 1,sp);
	}
}

#define max(a,b)	((a) > (b) ? (a) : (b))

int countlevels(rbnode *p, rbnode *nil)
{
	return p != nil ? max(countlevels(p->left,nil),countlevels(p->right,nil)) + 1 : 0;
}

void printtree(rbtree *p)
{
	int lev,i,j;
	struct spaces *q;

	struct spaces **sp = calloc(lev = countlevels(p->root,p->nil),sizeof(struct spaces *));

	for(i = 0; i < lev; i++)
		sp[i] = createlist();	
	levelvisit(p->root,p->nil,0,sp);
	for(i = 0; i < lev; i++) {
		j = 0;
		for(q = sp[i]->next;q != sp[i];q = q->next,j++) {
			for(;j < q->col;j++) 
				printf("      ");
			printf("[%2d|%c]",q->ptr->v,q->ptr->c == red ? 'R' : 'B' );
		}
		putchar('\n');
	}	
	for(i = 0; i < lev; i++)
		destroylist(sp[i]);
	free(sp);
}

