#include <stdio.h>
#include <stdlib.h>
#include "searchtree.h"

struct spaces {
	int col;
	searchtree *ptr;
	struct spaces *prev, *next;
};

struct spaces *createlist(void)
{
    struct spaces *q = malloc(sizeof(struct spaces));

    if(!q) {
        fprintf(stderr,"Errore di allocazione nella creazione della lista\n");
        exit(-1);
    }
    q->next = q->prev = q;
    return q;
}

void deletelist(struct spaces *q)
{
	q->prev->next = q->next;
	q->next->prev = q->prev;
	free(q);	
}

void destroylist(struct spaces *p)
{
	while(p->next != p)
		deletelist(p->next);
	free(p);
}

void insertatend(struct spaces *p, searchtree *ptr, int col)
{
	struct spaces *q = malloc(sizeof(struct spaces));

	if(!q) {
		fprintf(stderr,"Errore nell'allocazione del nuovo elemento\n");
		exit(-1);	
	}
	q->ptr = ptr;
	q->col = col;
	q->prev = p->prev;
	p->prev->next = q;
	p->prev = q;
	q->next = p;
}

void levelvisit(searchtree *p, int lev, struct spaces **sp)
{
	static int count = 0;

	if(!lev) count = 0;
	if(p)	{
		levelvisit(p->left,lev + 1,sp);
		insertatend(sp[lev],p,count++);
		levelvisit(p->right,lev + 1,sp);
	}
}

#define max(a,b)	((a) > (b) ? (a) : (b))

int countlevels(searchtree *p)
{
	return p ? max(countlevels(p->left),countlevels(p->right)) + 1 : 0;
}

void printtree(searchtree *p)
{
	int lev,i,j;
	struct spaces *q;

	struct spaces **sp = calloc(lev = countlevels(p),sizeof(struct spaces *));

	for(i = 0; i < lev; i++)
		sp[i] = createlist();	
	levelvisit(p,0,sp);
	for(i = 0; i < lev; i++) {
		j = 0;
		for(q = sp[i]->next;q != sp[i];q = q->next,j++) {
			for(;j < q->col;j++) 
				printf("    ");
			printf("[%2d]",q->ptr->v);
		}
		putchar('\n');
	}	
	for(i = 0; i < lev; i++)
		destroylist(sp[i]);
	free(sp);
}
